# -*- coding: utf-8 -*-

from django.shortcuts import get_object_or_404, render_to_response
from django.template import RequestContext
from django.contrib.auth.decorators import login_required
from django.http import HttpResponseRedirect
from django.utils.translation import ugettext_lazy as _
from django.core.urlresolvers import reverse_lazy
from django.shortcuts import redirect
from django.http import HttpResponseRedirect
from django.contrib.sites.models import Site
from django.template.defaultfilters import slugify
from datetime import datetime
from django.http import Http404
import urllib
from reversion.helpers import generate_patch_html
import reversion

import models as mymodels
import forms as myforms

from django.conf import settings as conf

from django.views.generic import DetailView, FormView, UpdateView, DeleteView


class PageDetailView(DetailView):

    template_name = "rwiki/default.html"
    context_object_name = "page"

    def __init__(self):
        pass

    def get(self, request, *args, **kwargs):
        try:
            self.object = self.get_object()
        except Http404:
            if (self.kwargs['slug']):
                return HttpResponseRedirect(reverse_lazy('app_wiki-add') +
                    "?%s" % urllib.urlencode({'ruta': self.kwargs['slug']}))
            else:
                return redirect('app_wiki-add')
        context = self.get_context_data(object=self.object)
        return self.render_to_response(context)

    def get_object(self):
        if (not self.kwargs['slug']):
            self.kwargs['slug'] = 'start'
        try:
            obj = mymodels.Page.objects.get(slug=self.kwargs['slug'], status=1)
        except:
            try:
                obj = mymodels.Page.objects.get(slug=self.kwargs['slug']+'/start', status=1)
            except:
                raise Http404
        if obj.public:
            return obj
        else:
            if self.request.user.is_superuser:
                return obj
            else:
                raise Http404
        #return obj
        #return get_object_or_404(mymodels.Page, slug=self.kwargs['slug'], status=1)

    def get_context_data(self, **kwargs):
        context = super(PageDetailView, self).get_context_data(**kwargs)
        try:
            history = reversion.get_unique_for_object(self.object)
            last = history[0]
            for h in history:
                h.patch = generate_patch_html(last, h, "text", cleanup="semantic")
        except:
            history = ''

        breadcrumb = self.object.slug.split('/')
        breadcrumbs = {}
        for b in breadcrumb[:-1]:
            try:
                bread = mymodels.Page.objects.get(slug=b, status=1)
                breadcrumbs[b] = bread.slug
            except:
                try:
                    bread = mymodels.Page.objects.get(slug="%s/start" % b, status=1)
                    breadcrumbs[b] = bread.slug
                except:
                    breadcrumbs[b] = ''
        context.update({
            'history': history,
            'breadcrumbs': breadcrumbs,
            'last': breadcrumb[-1],
        })
        return context

    def get_template_names(self):
        """
         `get_object` has already set this attribute, so we pick up the
         template from the model. I it doesn't exists we pick up the
         super name.
        """
        try:
            templates = [self.object.template]
            return templates + super(PageDetailView, self).get_template_names()
        except:
            return super(PageDetailView, self).get_template_names()


class PageAdd(FormView):

    """
    Class Based View, add a new page.
    """

    form_class = myforms.PageForm
    template_name = 'rwiki/page_form.html'

    def get_initial(self):
        try:
            return { 'slug': self.request.GET['ruta'] }
        except:
            pass

    def get_success_url(self):
        return reverse_lazy('app_wiki-detail', args=(self.returnpath, ))

    def dispatch(self, *args, **kwargs):
        return super(PageAdd, self).dispatch(*args, **kwargs)

    def form_valid(self, form):
        self.returnpath = form.cleaned_data['slug']
        newpage = mymodels.Page( title = form.cleaned_data['title'],
                                 slug = form.cleaned_data['slug'],
                                 created = datetime.now(),
                                 text = form.cleaned_data['text'],
                                 hits = 1,
                                 status = form.cleaned_data['status'],
                                 template = form.cleaned_data['template'],
                                 owner = self.request.user)
        with reversion.create_revision():
            newpage.save()
            reversion.set_user(self.request.user)
            reversion.set_comment(form.cleaned_data['comment'])
        return super(PageAdd, self).form_valid(form)

    def get_context_data(self, **kwargs):
        context = super(PageAdd, self).get_context_data(**kwargs)
        return context


class PageUpdate(UpdateView):

    """
    Class Based View, add a new page.
    """

    form_class = myforms.PageForm
    template_name = 'rwiki/page_form.html'

    def get_success_url(self):
        return reverse_lazy('app_wiki-detail', args=(self.kwargs['slug'], ))

    def get_object(self):
        self.obj = get_object_or_404(mymodels.Page, slug=self.kwargs['slug'], status=1)
        return self.obj

    def form_valid(self, form):
        with reversion.create_revision():
            reversion.set_user(self.request.user)
            reversion.set_comment(form.cleaned_data['comment'])
            form.instance.save()
        return super(PageUpdate, self).form_valid(form)

    def get_context_data(self, **kwargs):
        context = super(PageUpdate, self).get_context_data(**kwargs)
        return context


def revert(request, objid, revid):
    page = get_object_or_404(mymodels.Page, id=objid, status=1)
    history = reversion.get_unique_for_object(page)
    for h in history:
        if str(h.revision_id) == str(revid):
            h.revision.revert()
    return redirect('app_wiki-detail', page.slug)


class PageDel(DeleteView):

    """
    Class Based View, delete page.
    """

    form_class = myforms.PageForm
    context_object_name = "page"

    def get(self, request, *args, **kwargs):
        self.obj = get_object_or_404(mymodels.Page, slug=self.kwargs['slug'])
        self.obj.delete()
        return redirect('app_wiki-detail', 'start')


# -*- coding: utf-8 -*-
from django import forms
import models as mymodels
from django.core.files.images import get_image_dimensions

class PageForm(forms.ModelForm):

    comment = forms.CharField(required=False)

    class Meta:
        model = mymodels.Page
        exclude = ('hits', 'owner')



# -*- coding: utf-8 -*-

from django.contrib import admin
import models as mymodels
import forms as myforms
import socket
import reversion

class AdminPage(reversion.VersionAdmin):
    #exclude = ('user',)

    # http://www.userlinux.net/django-los-campos-tipo-slug.html
    prepopulated_fields = { 'slug': ['title'] }

class AdminCategory(admin.ModelAdmin):
    prepopulated_fields = { 'slug': ['title'] }

admin.site.register(mymodels.Page, AdminPage)



# -*- coding: utf-8 -*-

from django.conf.urls.defaults import *
from django.contrib.auth.decorators import login_required, permission_required
from rwiki.views import PageDetailView, PageAdd, PageUpdate, PageDel, revert

# Llamamos al registro a traves del formulario RegisterForm que hemos creado
urlpatterns = patterns('',
    url(r'^$',                              PageDetailView.as_view(), {'slug': 'start'}, 'app_wiki-detail'),
    url(r'^add/$',                          permission_required('rwiki.add_page')(PageAdd.as_view()), {}, 'app_wiki-add'),
    url(r'^revert/(?P<objid>\d+)/(?P<revid>\d+)/$', permission_required('rwiki.reverse')(revert), {}, name='app_wiki-revert'),
    url(r'^(?P<slug>[-\w/]+?)/del/$',      permission_required('rwiki.del_page')(PageDel.as_view()), {}, 'app_wiki-del'),
    url(r'^(?P<slug>[-\w/]+?)/edit/$',      permission_required('rwiki.change_page')(PageUpdate.as_view()), {}, 'app_wiki-edit'),
    url(r'^(?P<slug>[-\w/]+?)/$',           PageDetailView.as_view(), {}, 'app_wiki-detail'),
)



# -*- coding: utf-8 -*-

"""
Models
"""

import time
import socket
from django.db import models
from django.contrib.auth.models import User
from django.utils.translation import ugettext_lazy as _
from django.contrib.contenttypes import generic
from django.template.defaultfilters import slugify
from django.conf import settings as conf


class Page(models.Model):

    """
    Page model
    """

    title = models.CharField(_(u'Title'), max_length=255)
    slug = models.CharField(_(u'Slug'), max_length=200)
    created = models.DateTimeField(auto_now_add=True, verbose_name=_(u'Created'))
    text = models.TextField(_(u'Text'))
    hits = models.IntegerField(_(u'Hits'), default=0)
    status =  models.BooleanField(_(u'Status'), default=True)
    public =  models.BooleanField(_(u'Public'), default=True, help_text="Public pages are shown to everyone, privates only to admin user role")
    template = models.CharField(_(u'Render template'), max_length=100,
        choices=conf.RWIKI_TEMPLATES, blank=True)
    owner = models.ForeignKey(User, verbose_name=_(u'Owner'),
                              blank=True, null=True, related_name='owner')

    def __unicode__(self):
        return self.slug

    class Meta:
        verbose_name = _(u'Page')
        verbose_name_plural = _(u'Pages')
        permissions = (
            ("reverse", "Can reverse a version"),
        )



# -*- coding: utf-8 -*-
from setuptools import setup, find_packages

setup(
    name='django-rwiki',
    version='0.0.2',
    author=u'Oscar M. Lage Guitian',
    author_email='r0sk10@gmail.com',
    #packages=['rwiki'],
    packages = find_packages(),
    include_package_data = True,
    package_data = {'': ['rwiki/templates', 'rwiki/static','rwiki/fixtures',]},
    url='http://bitbucket.org/r0sk/django-rwiki',
    license='BSD licence, see LICENSE file',
    description='Yet another Django Wiki App',
    zip_safe=False,
    long_description=open('README.rst').read(),
    install_requires=[
        "Django < 1.5",
        "South == 0.7.5",
        "PIL == 1.1.7",
        "django-filebrowser-no-grappelli == 3.1.1",
        "sorl-thumbnail == 11.12",
        "django-compressor == 1.1.2",
        "django-reversion == 1.6.3",
        "diff-match-patch==20120106",
    ],
    keywords = "django application wiki",
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
)
